/*
//	EventLoop
//	A simple event loop to take care of windows and menus
//	many of the window events are handled in document.c,
//	so make changes there instead of here.
//
//	By Brigham Stevens
//	 1992, 1993 Apple Computer, Inc.
//
*/
#include "GlobalDefs.h"
#include "AppleEvents.h"

Boolean		gDone;
Boolean		gBackgroundFlag;
EventRecord gEvent;

long GetBackTime(void);

void InitEventGlobals(void)
{
	gDone = false;
	gBackgroundFlag = false;
}

void DoCloseWindow(WindowPtr theWindow)
{
	if(TrackGoAway(theWindow,gEvent.where)) {
		CloseDocument(theWindow);
	}
}

void DoClickInContent(WindowPtr theWindow)
{
	
	if(theWindow!=FrontWindow()) {
		SelectWindow(theWindow);
	} else {
		ClickDocument(theWindow);
	}
}

void DoDragWindow(WindowPtr theWindow)
{
	DragWindow(theWindow,gEvent.where,&screenBits.bounds);
}


void DoGrowWindow(WindowPtr theWindow)
{
	long	newSize;
	int		newHeight,newWidth;
	Rect	growLimitSizes;
	
	SetPort(theWindow);
	InvalRect(&theWindow->portRect);
	
	growLimitSizes.top = 40;			/* min height */
	growLimitSizes.bottom = 32767;		/* max height */
	growLimitSizes.left = 40; 			/* min width */
	growLimitSizes.right = 32767;		/* max width */
	
	newSize = GrowWindow(theWindow,gEvent.where,&growLimitSizes);
	newHeight = HiWord(newSize);
	newWidth = LoWord(newSize);
	SizeWindow(theWindow,newWidth,newHeight,TRUE);
}

void DoZoom(WindowPtr theWindow, int part)
{
	GrafPtr savePort;
	
	GetPort(&savePort);
	SetPort(theWindow);
	
	if(TrackBox(theWindow,gEvent.where,part)) {
		ZoomWindow(theWindow,part,true);
	}
	
	SetPort(savePort);
}



void DoMenu(long msel)
/*
	This is the interface to our menu dispatcher
	All menu commands pass through here after they
	are selected by MenuKey or MenuSelect.
*/
{
	short item,menu;
	
	item = msel;		// C will truncate and only get the low word
	menu = msel >> kLow16Bits;	// the menu number is in the high word
	
	MenuDispatch(menu, item);
	
	HiliteMenu(0);						/* remove menu title hiliting */
}

void DoKey(void)
/*
	Handles key presses
	if the command key is down this takes care of menu selecting.
	Otherwise it passes the key to the document window
*/
{
	WindowPtr	fWindow;
		
	if((gEvent.modifiers & cmdKey) == FALSE) {
		fWindow = FrontWindow();
		if( fWindow != nil)
			KeyDocument(fWindow);
	} else {
		AdjustMenus();
		DoMenu(MenuKey(gEvent.message & charCodeMask));	
	}
}


void DoUpdate(void)
{
	WindowPtr	updateWindow;
	GrafPtr		savePort;
	
	GetPort(&savePort);						/* save current port */
	
	updateWindow=(WindowPtr)gEvent.message;	/* get windowPtr from event msg */
	SetPort(updateWindow);
	BeginUpdate(updateWindow);						

	UpdateDocument(updateWindow);

	EndUpdate(updateWindow);
		
	SetPort(savePort);
}



void DoActivate(void)
{
	PaletteHandle pal;
	
	if(gEvent.modifiers & activeFlag)
	{
		pal = GetPalette( (WindowPtr)gEvent.message);
		if(pal)
			ActivatePalette((WindowPtr)gEvent.message );
	}
}

void DoOSEvent(void)
/*
	Check the high 8 bits of the event message to see if we have
	a suspend resume message, and set our global flag indicating if
	we are in the background.
	
	A way to do this without a flag is to use the process manager when we
	need to know.  If GetCurrentProcess's PSN does not match the GetFrontProcess's PSN
	then we are in the background.
*/
{
	if( (gEvent.message >> kLow24Bits) == suspendResumeMessage)
		gBackgroundFlag = !(gEvent.message & resumeFlag);
}

void DoClick(void)
{
	WindowPtr	theWindow;
	short		part;
	
	part = FindWindow(gEvent.where, &theWindow);
	
	switch(part) {
		case inDesk:		
		break;
		case inMenuBar:		AdjustMenus();
							DoMenu(MenuSelect(gEvent.where));
		break;
		case inSysWindow:	SystemClick(&gEvent,theWindow);
		break;
		case inContent:		DoClickInContent(theWindow);
		break;
		case inDrag:		DoDragWindow(theWindow);
		break;
		case inGrow:		DoGrowWindow(theWindow);
		break;
		case inGoAway:		DoCloseWindow(theWindow);
		break;
		case inZoomIn:
		case inZoomOut:		DoZoom(theWindow,part);
		break;
		default:			
		break;
	}
}


void MainEvent(void)
{

	if(WaitNextEvent(everyEvent,&gEvent,GetBackTime(),nil)) {
		switch(gEvent.what) {
			case nullEvent:			IdleDocument(FrontWindow());			
			break;
			case mouseDown:			DoClick();			
			break;
			case mouseUp:		 								
			break;
			case keyDown:			DoKey();				
			break;
			case keyUp:			 								
			break;
			case autoKey:			DoKey();				
			break;
			case updateEvt:			DoUpdate();			
			break;
			case diskEvt:		 								
			break;
			case activateEvt:		DoActivate();			
			break;
			case networkEvt:									
			break;
			case driverEvt:		 								
			break;
			case app1Evt:		 								
			break;
			case app2Evt:		 								
			break;
			case app3Evt:		 								
			break;
			case osEvt:				DoOSEvent();			
			break;
			case kHighLevelEvent:	DoHighLevelEvent(&gEvent);
			default:											
			break;
		}
	}
}


